<?php

namespace Drupal\digest;

use Drupal\user\Entity\User;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Config form builder for Digest module.
 */
class UserSettings extends ConfigFormBase {

  protected $user;

  /**
   * {@inheritdoc}
   */
  public function __construct($current_request) {
    $this->user = $current_request->getParameters()->get('user');
    if (is_numeric($this->user)) {
      $this->user = User::load($this->user);
    }
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('current_route_match')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    $form = Digest::userSettingsForm($this->user);
    $form['actions']['submit'] = [
      '#type' => 'submit',
      '#value' => t('Save'),
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    Digest::userSettingsFormSubmit($form, $form_state, $this->user);
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'digest_user_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function getEditableConfigNames() {
    return [];
  }

  /**
   * Router access callback.
   *
   * @todo why is this here?
   */
  public function access() {
    return AccessResult::allowed();
  }

}
