<?php

namespace Drupal\cforge\Installer;

use Drupal\taxonomy\Entity\Term;
use Drupal\Core\Render\Element;
use Drupal\user\Entity\User;
use Drupal\taxonomy\Entity\Vocabulary;
use Drupal\Core\Form\FormBase;
use Drupal\Core\Form\FormStateInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

ini_set('max_execution_time', 150);

/**
 * Configuration options for each flavour.
 */
class Setup2 extends FormBase {

  /**
   * Constructor.
   */
  public function __construct($module_installer, $account_switcher) {
    $this->installer = $module_installer;
    $this->accountSwitcher = $account_switcher;
    // Contains cforge_setup2_form() and cforge_setup2_form_submit().
    $flavour = cforge_get_flavour();
    module_load_install($flavour);
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('module_installer'),
      $container->get('account_switcher')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'cforge_setup_2_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state) {
    // This uses the machine name.
    $form['#title'] = t('Setup @flavour', ['@flavour' => cforge_get_flavour()]);

    // Tried to put this in Setup1 but couldn't work out how to propagate the
    // result until Setup2.
    $form['data_content'] = [
      '#title' => t('Prepare the site with'),
      '#type' => 'radios',
      '#options' => [
        'example' => t('Example data'),
        'import' => t('Import Tools'),
        'none' => t('Nothing'),
      ],
      '#weight' => 5,
      'example' => [
        '#disabled' => !system_rebuild_module_data()['devel_generate'],
      ],
      '#required' => TRUE,
    ];
    module_load_install('cforge');
    $demo_data_class = cforge_demo_data_class_name();
    $form['neighbourhoods'] = [
      '#title' => $this->t('Neighbourhoods'),
      '#description' => $this->t('Limited list of possible neighbourhoods, one per line, in order. Leave blank if you plan to import data'),
      '#type' => 'textarea',
      '#default_value' => implode("\r\n", $demo_data_class::$hoods),
      '#weight' => 8,
      '#required' => 1,
      '#states' => [
        'visible' => [
          ':input[name=data_content]' => ['value' => 'none'],
        ],
      ],
    ];

    $vocab_names = array_keys($demo_data_class::vocabs());
    $form['default_categories'] = array(
      '#title' => $this->t('Preload ad categories'),
      '#description' => $this->t('Can be modified later'),
      '#type' => 'radios',
      '#options' => [0 => $this->t('- None -')] + array_combine($vocab_names, $vocab_names),
      '#default_value' => key($vocab_names),
      '#weight' => 9,
      '#states' => [
        'visible' => [
          ':input[name=data_content]' => ['value' => 'none'],
        ],
      ],
    );

    if (function_exists('cforge_setup2_form_elements')) {
      $form += cforge_setup2_form_elements();
    }
    $form['submit'] = [
      '#type' => 'submit',
      '#value' => 'submit',
      '#weight' => 20,
    ];
    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $data_content = $form_state->getValue('data_content');
    global $install_state;
    $install_state['example'] = $data_content == 'example';
    module_load_install('cforge');
    $demo_data_class = cforge_demo_data_class_name();
    $this->accountSwitcher->switchTo(User::load(1));

    if ($data_content == 'example') {
      \Drupal::service('module_installer')->install(
        ['views_ui',
          'field_ui',
          'contextual',
          'config',
          'admin_toolbar_tools',
          'devel',
          'devel_generate',
        ]
      );
      \Drupal::configFactory()
        ->getEditable('views.settings')
        ->set('ui.show.sql_query.enabled', 1)
        ->set('ui.always_live_preview', 1)
        ->set('ui.show.advanced_column', 1)
        ->set('ui.show.master_display', 1)
        ->save();


      if ($data_content == 'example') {
        $hoods = $demo_data_class::$hoods;
      }
      else {
        $hoods = array_unique(explode("\n", trim($form_state->getValue('neighbourhoods'))));
      }
      $demo_data_class::createNeighbourhoods($hoods);
    }
    elseif ($data_content == 'import') {
      \Drupal::service('module_installer')->install(['cforge_import']);
      cforge_setup2_form_submit($form, $form_state);
    }

    $terms = \Drupal::entityTypeManager()
      ->getStorage('taxonomy_term')
      ->loadByProperties(['vid' => SMALLAD_CATEGORIES]);
    if ($form_state->getValue('default_categories')) {
      // Delete any existing terms in the vocab (one is installed by default)
      foreach ($terms as $term) {
        $term->delete();
      }
      $vocab = Vocabulary::load(SMALLAD_CATEGORIES);
      // Now import the terms into the vocab.
      $this->vocabImportRecursive(
        $vocab,
        $demo_data_class::vocabs()[$form_state->getValue('default_categories')]
      );
      // Thie hierarchy property.
      $vocab->save();
    }
    $this->accountSwitcher->switchBack();
  }

  /**
   * Import the terms in a vocabulary or a level of a vocab.
   */
  public function vocabImportRecursive(Vocabulary $vocab, array $termArray, $parent = 0) {
    $w = $hierarchy = 0;
    foreach (Element::Children($termArray) as $name) {
      $props = [
        'name' => $name,
        'weight' => $w++,
        'vid' => $vocab->id(),
        'parent' => $parent,
        'format' => 'plain_text',
      ];
      $term = Term::Create($props);
      $term->save();
      if ($termArray['#children']) {
        $vocab->setHierarchy(TAXONOMY_HIERARCHY_SINGLE);
        $this->vocabImportRecursive($vocab, $termArray[$name], $term->tid);
      }
    }
  }

}
