<?php

namespace Drupal\cforge\Plugin\Block;


use Drupal\user\Entity\UserInterface;
use Drupal\Core\Access\AccessResult;
use Drupal\Core\Block\BlockBase;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\Core\Session\AccountInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Prompts a new user to enter the referrer.
 *
 * @Block(
 *   id = "nag_referrer",
 *   admin_label = @Translation("Nag new user to enter referrer"),
 *   category = @Translation("Hamlets")
 * )
 */
class NagReferrer extends BlockBase implements ContainerFactoryPluginInterface {

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container, array $configuration, $plugin_id, $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition
    );
  }

  /**
   * {@inheritdoc}
   */
  protected function blockAccess(AccountInterface $account) {
    $result = (\Drupal::keyValue('new_user_reference')->get($account->id()) && cforge_referral())
      ? AccessResult::allowed()
      : AccessResult::forbidden();
    // @todo how do we to clear this cache context?
    return $result->cachePerUser();
  }

  /**
   * {@inheritdoc}
   */
  public function build() {
    $form = \Drupal::formBuilder()->getForm('Drupal\cforge\Forms\ChooseReferrer', \Drupal::currentUser()->id());
    return $form;
  }

}

/**
 * Implements hook_user_insert().
 */
function cforge_user_insert(UserInterface $account) {
  // Prevent this from applying to imports.
}
